(*
    File locking utility - Main form
    Copyright (c) 1999-2003,2004 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)
unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ComCtrls, ImgList;

type
  TformMain = class(TForm)
    dlgOpen: TOpenDialog;
    lstLocked: TListView;
    mainMenu: TMainMenu;
    mnuFile: TMenuItem;
    mnuAdd: TMenuItem;
    mnuRemove: TMenuItem;
    N1: TMenuItem;
    mnuExit: TMenuItem;
    mnuAbout: TMenuItem;
    N2: TMenuItem;
    mnuLock: TMenuItem;
    mnuUnlock: TMenuItem;
    imlLocked: TImageList;
    popupList: TPopupMenu;
    popLock: TMenuItem;
    popUnlock: TMenuItem;
    popRemove: TMenuItem;
    N3: TMenuItem;
    popAdd: TMenuItem;
    barStatus: TStatusBar;
    procedure mnuAboutClick(Sender: TObject);
    procedure lstLockedResize(Sender: TObject);
    procedure mnuAddClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure lstLockedChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure mnuLockClick(Sender: TObject);
    procedure mnuUnlockClick(Sender: TObject);
    procedure mnuRemoveClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure mnuExitClick(Sender: TObject);
    procedure lstLockedDblClick(Sender: TObject);
  private
    { Private declarations }
    function IsLocked( F: TListItem ): boolean;
    procedure AddItem( Name: string );
    procedure LockItem( F: TListItem );
    procedure LockSelected;
    procedure OnDropFiles( var M: TMessage ); message WM_DROPFILES;
    procedure UnlockItem( F: TListItem );
    procedure UnlockSelected;
    procedure UpdateFileMenu;
    procedure UpdateStatus( NumLocked, NumUnlocked: integer );
  public
    { Public declarations }
  end;

var
  formMain: TformMain;

implementation

uses ShellAPI;

{$R *.DFM}

procedure TformMain.mnuAboutClick(Sender: TObject);
begin
  MessageBox( Handle,
    'Copyright (c) 1999-2003 Alessandro Scotti',
    'About',
    MB_OK or MB_ICONINFORMATION );
end;

procedure TformMain.lstLockedResize(Sender: TObject);
begin
  with lstLocked do begin
    Columns[0].Width := (Width * 80) div 100;
    Columns[1].Width := Width - 4 - Columns[0].Width;
  end;
end;

function GetFileSizeByName( FileName: string ): integer;
var
  SR: TSearchRec;
begin
  Result := 0;
  if( FindFirst( FileName, faAnyFile, SR ) = 0 ) then begin
    Result := SR.Size;
    FindClose( SR );
  end;
end;

procedure TformMain.AddItem( Name: string );
var
  LI: TListItem;
  SZ: integer;
  ML: string;
begin
  LI := lstLocked.Items.Add;
  LI.Caption := Name;
  LI.Data := nil;
  LI.ImageIndex := 0;
  SZ := GetFileSizeByName( Name );
  ML := IntToStr( SZ );
  if( SZ > 100000 ) then ML := IntToStr( (SZ+511) div 1024 ) + 'K';
  if( SZ > 10000000 ) then ML := IntToStr( (SZ+524287) div (1024*1024) ) + 'M';
  LI.SubItems.Add( ML );
end;

procedure TformMain.mnuAddClick(Sender: TObject);
var
  ID: integer;
begin
  if( dlgOpen.Execute ) then begin
    lstLocked.Items.BeginUpdate;
    for ID:=1 to dlgOpen.Files.Count do begin
      AddItem( dlgOpen.Files[ID-1] );
    end;
    lstLocked.Items.EndUpdate;
    UpdateStatus( -1, -1 );
  end;
end;

procedure TformMain.UpdateFileMenu;
var
  SL: boolean;
begin
  SL := lstLocked.SelCount > 0;
  mnuLock.Enabled := SL;
  mnuUnlock.Enabled := SL;
  mnuRemove.Enabled := SL;
  popLock.Enabled := SL;
  popUnlock.Enabled := SL;
  popRemove.Enabled := SL;
end;

function TformMain.IsLocked( F: TListItem ): boolean;
begin
  Result := F.Data <> nil;
end;

procedure TformMain.LockItem( F: TListItem );
var
  HF: THandle;
begin
  if( not IsLocked(F) ) then begin
    HF := CreateFile( PChar(F.Caption),
      GENERIC_READ,
      0,
      nil,
      OPEN_EXISTING,
      0,
      0 );
    if( HF <> INVALID_HANDLE_VALUE ) then begin
      F.ImageIndex := 1;
      F.Data := pointer(HF);
    end
    else begin
      F.ImageIndex := 2;
    end;
  end
  else begin
    F.ImageIndex := 1;
  end;
end;

procedure TformMain.UnlockItem( F: TListItem );
begin
  if( IsLocked(F) ) then begin
    if( CloseHandle( THandle(F.Data) ) ) then begin
      F.ImageIndex := 0;
      F.Data := nil;
    end;
  end
  else begin
    F.ImageIndex := 0;
  end;
end;

procedure TformMain.LockSelected;
var
  ID: integer;
  LI: TListItem;
  NN: integer;
begin
  NN := 0;
  for ID:=1 to lstLocked.Items.Count do begin
    LI := lstLocked.Items[ID-1];
    if( LI.Selected ) then LockItem( LI );
    if( IsLocked(LI) ) then Inc( NN );
  end;
  UpdateStatus( NN, -1 );
end;

procedure TformMain.UnlockSelected;
var
  ID: integer;
  LI: TListItem;
  NN: integer;
begin
  NN := 0;
  for ID:=1 to lstLocked.Items.Count do begin
    LI := lstLocked.Items[ID-1];
    if( LI.Selected ) then UnlockItem( LI );
    if( not IsLocked(LI) ) then Inc( NN );
  end;
  UpdateStatus( -1, NN );
end;

procedure TformMain.FormClose(Sender: TObject; var Action: TCloseAction);
var
  ID: integer;
begin
  for ID:=1 to lstLocked.Items.Count do begin
    UnlockItem( lstLocked.Items[ID-1] );
  end;
end;

procedure TformMain.lstLockedChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
  if( Change = ctState ) then UpdateFileMenu;
end;

procedure TformMain.mnuLockClick(Sender: TObject);
begin
  LockSelected;
end;

procedure TformMain.mnuUnlockClick(Sender: TObject);
begin
  UnlockSelected;
end;

procedure TformMain.mnuRemoveClick(Sender: TObject);
var
  ID: integer;
  LI: TListItem;
begin
  UnlockSelected;
  lstLocked.Items.BeginUpdate;
  for ID:=lstLocked.Items.Count downto 1 do begin
    LI := lstLocked.Items[ID-1];
    if( LI.Selected ) then LI.Delete;
  end;
  lstLocked.Items.EndUpdate;
end;

procedure TformMain.OnDropFiles( var M: TMessage );
var
  HD: THandle;
  NM: array[ 1..MAX_PATH ] of char;
  ID: integer;
begin
  HD := M.WParam;
  for ID := 0 to DragQueryFile( HD, $FFFFFFFF, nil, 0 )-1 do begin
    DragQueryFile( HD, ID, @NM, sizeof(NM) );
    AddItem( StrPas(@NM) );
  end;
  DragFinish( HD );
  M.Result := 0;
  UpdateStatus( -1, -1 );
end;

procedure TformMain.FormCreate(Sender: TObject);
begin
  DragAcceptFiles( Handle, true );
end;

procedure TformMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
  LK: boolean;
  ID: integer;
begin
  LK := false;
  for ID:=1 to lstLocked.Items.Count do begin
    LK := LK or IsLocked(lstLocked.Items[ID-1]);
  end;
  if( LK ) then begin
    ID := MessageBox( Handle,
      'Closing the application will unlock all files. Continue?',
      'Warning',
      MB_YESNOCANCEL or MB_ICONQUESTION );
    CanClose := ID = ID_YES;
  end;
end;

procedure TformMain.mnuExitClick(Sender: TObject);
begin
  Close;
end;

procedure TformMain.UpdateStatus( NumLocked, NumUnlocked: integer );
var
  ID: integer;
begin
  if( NumLocked < 0 )and( NumUnlocked < 0 ) then begin
    NumLocked := 0;
    for ID:=1 to lstLocked.Items.Count do
      if( IsLocked( lstLocked.Items[ID-1] ) ) then Inc( NumLocked );
    NumUnlocked := lstLocked.Items.Count - NumLocked;
  end;
  if( NumLocked < 0 ) then NumLocked := lstLocked.Items.Count - NumUnlocked;
  if( NumUnlocked < 0 ) then NumUnlocked := lstLocked.Items.Count - NumLocked;
  barStatus.Panels[0].Text := 'Files: ' + IntToStr(lstLocked.Items.Count);
  barStatus.Panels[1].Text := 'Locked: ' + IntToStr(NumLocked);
  barStatus.Panels[2].Text := 'Unlocked: ' + IntToStr(NumUnlocked);
end;

procedure TformMain.lstLockedDblClick(Sender: TObject);
begin
  if( lstLocked.SelCount = 1 ) then begin
    if( IsLocked( lstLocked.Selected ) ) then begin
      UnlockSelected;
    end
    else begin
      LockSelected;
    end;
  end;
end;

end.
